<?php

# httpmanage 0.4.3, 2000-05-18, tool to manage two versions of WWW files
# Copyright (C) 1999,2000 Daniel Naber <dnaber@mini.gt.owl.de>

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# WARNING: see README for security issues

# KNOWN PROBLEMS:
#	change state to "online" doesn't kill possible warning, you
#		really have to upload the file to get rid of the warning
#	special characters in links (e.g. filename) are not yet encoded
#	you cannot use "'" as a string for username, logfile entry etc.
#	the timezone is chopped off to compare dates with postgres,
#		I hope this works for timezone == GMT
#	you should disable httpmanage if checking your site
#		with linbot or you will get hundreds of errors

# TODO:
#	see "fixme" and "todo" in the source
#	only show "download now" link if there is actually something to download
#	automatically add '/' at end of name if directory is managed
#	automatically add '/' at beginning of filename if it's missing
#	more back links (using javascript where available?)
#	add useful link on "view changes" page
#	don't link binary files directly but link with admin.php3?filename=binaryfile ?
#	use ssh even with a password (expect-script)
#	make sure the generated pages are valid HTML (set DTD etc.)
#	use javascript to jump to input fields (onLoad)
#	use CSS to layout the pages?
#	use cookies to remember username for upload
#	how to get rid of all the warnings when accessing non-existing fields?
#		(the warnings only show up with PHP "error_reporting = 8")
#	optionally send expires header for the logfile view?

# CHANGES (only the important or recent ones):
#	1999-09-12: first version
#	1999-09-25: first public version: 0.1
#	1999-09-27: released version 0.2
#	1999-10-25: released version 0.3
#	1999-11-09: avoid some warnings thanks to isset();
#	1999-11-13: started moving to phplib for database access
#	2000-02-27: differences in DOS/Unix line endings may now be ignored by the diff command;
#		"/tmp" isn't used anymore but "tmp" (for security reasons)
#	2000-03-01: using flush() between uploads of multiple files
#	2000-03-04: "view changes" is now called "history" and more powerful; cleanups;
#		released as version 0.4
#	2000-03-05: added DTD; key/value pairs now use "&amp;" as delimiter instead of "&"
#		(yes, this is correct! It's necessary for the documents to validate)
#	2000-03-07: fixed the broken "history" link, fixed typo in $suffix_picture
#	2000-03-08: fixed a broken SQL query in "history"
#	2000-05-18: fixed a typo that made it impossible to upload several files at once

# --- configuration -----------------------------------------

# path to the httpmanage directory (no trailing slash):
$path = "/usr/local/httpd/htdocs/testsite/httpmanage";

# here you can select the external config file that will be chosen,
# depending on the path/filename. the array's second argument is the project's name:
$config["/httpmanage_html/"] = array("$path/config_httpmanage.php3", "httpmanage");
$config["/testsite/"] = array("$path/config_testsite.php3", "testsite");
# you may add more configuration files here...
# if you only have one config use this:
# config[".*"] = array("/home/www/testsite/httpmanage/config_mysite.php3", "my site");

# turn on/off the use of a database (features 'change state' and 'changelog' will
# be disabled if set to 0):
$usedb = 1;
# warn about possibly wrong "online" status:
$datecheck = 1;

# which db to use (if any). set to "pgsql" to use PostgreSQL or "mysql" to use MySQL:
#$db_type = "pgsql";
$db_type = "mysql";

# here are some options that are mostly independent of the project. Still you may
# delete them here and put them in the external files above:

$checkcmd = "/usr/bin/weblint";			# command to check html syntax (use "" to disable command)
$diffcmd = "/usr/bin/diff -u";			# don't change -u if you want a color diff
$downloaddir = "tmp";				# DOWNLOADDIR in $downloadcmd will be replaced with this,
						# do not set to "/tmp" for security reasons ("tmp" is okay)!


# if you include admin.php3 via SSI, set this to 1, otherwise (e.g. if you do
# a php include()) set it to 0:
$ssi_include = 0;

# how to color the lines that differ between files:
$diff_old_start = '<font color="red">';
$diff_old_end = '</font>';
$diff_new_start = '<font color="green">';
$diff_new_end = '</font>';

# the script will stop if its hasn't finished after this number of seconds.
# Note that safe_mode in your php3.ini must be Off to make this work:
$timelimit = 5*60;

# the filenames are used to decide what we can do with that file (give regular expressions).
# anything that can be shown via <img...>:
$suffix_picture = '\.(jpeg|jpg|gif|png)$';
# anything that can be compared line by line with diff:
$suffix_html = '\.(html|htm|txt|php3|php)$';

# --- end of configuration -----------------------------------------

require("$path/phplib/db_$db_type.inc");
require("$path/phplib/ct_sql.inc");
require("$path/phplib/local.inc");

function action($file) {
	global $db, $cmd, $usedb, $script, $server, $onlineserver, $config, $cfgname, $ssi_include, $docroot, $suffix_html, $checkcmd, $datecheck, $db_type;
	$onlineurl = $onlineserver.urlfilter($file);

	if( (! $file || $file == $script) && $cmd != 'viewalllogs' && $cmd != 'putonlineall'
		&& $cmd != 'doputonlineall' && $cmd != 'start' ) {
		print "Error: admin.php3 needs 'filename' argument or must be started with ...?cmd=start";
		return;
	}
	if( ! file_exists("$docroot$file") ) {
		print "Error: '$docroot$file' does not exist";
		return;	
	}
	
	if( ! isset($cmd) ) {
		$directcall = 0;
		if( ! ereg($suffix_html, $file) ) {
			$directcall = 1;			# 1 = called directly, i.e. not included in a html/php file
			headline("manage file");
		}
		# this is what normally gets included in the html/php file:
		print "<!-- begin_of_httpmanage --><small><br><br>\n";
		if( $usedb ) {
			if( $db_type == "pgsql" ) {
				$db->query("SET Datestyle='ISO'");
			}
			$query = "	SELECT state.description, state.id AS stateid, files.date, files.id, files.filename, overview.fileid, overview.state
					FROM files, overview, state WHERE files.filename = '$file'
					AND files.id = overview.fileid
					AND overview.state = state.id";
			$db->query($query);
			$db->next_record();
			if( ! $db->f("state") ) {
				$state = "[no state]";
			}
			print "<strong>".$db->f("description");
			# check if the file has been modified since the last upload via httpmanage.
			# this way the user gets warned about modified files with a possibly wrong 'online' state:
			$filedate = stat("$docroot$file");
			$filedate = $filedate[9];		# date of last modification
			$filedate = date("Y-m-d H:i:s", $filedate);
			$stateid = $db->f("stateid");
			$date = $db->f("date");
			if( $db_type == "pgsql" ) {
				$date = substr($date, 0, strlen($date)-3);	# cut of timezone
			}
			if( $datecheck && isset($stateid) && $stateid == 0 && isset($date) && $filedate != $date ) {
				print "? <font color=\"red\">File has been changed $filedate</font>";
			}
			print ":</strong>\n";
		}
		print "$file in project '$cfgname'<br>";
		if( $usedb ) {
			print "<strong><a href=\"$script?cmd=changestate&amp;filename=$file&amp;statenow=".$stateid."&amp;directcall=$directcall\">change&nbsp;state</a></strong> /\n";
		}
		print "<strong><a href=\"$script?cmd=putonline&amp;filename=$file&amp;directcall=$directcall\">upload</a></strong>\n";
		if( ! $directcall && $checkcmd ) {
			print "/ <a href=\"$script?cmd=checksyntax&amp;filename=$file\">check&nbsp;syntax</a>\n";
		}
		global $enable_redirect_link;
		if( $enable_redirect_link ) {
			if( ! $ssi_include ) {		# just won't work with SSI
				print "/ <a href=\"$file?cmd=previewdev\">preview&nbsp;dev</a>\n";
			}
			print "/ <a href=\"$onlineurl\">view&nbsp;online</a>\n";
		}
		if( is_binary($file) ) {
			print "/ <a href=\"$script?cmd=diff&amp;filename=$file&amp;difftype=wheter\">diff&nbsp;dev/online</a>\n";
		} else {
			print "/ <a href=\"$script?cmd=diff&amp;filename=$file&amp;difftype=normal\">diff&nbsp;dev/online</a>\n";
		}
		if( $usedb ) {
			print " / <a href=\"$script?cmd=viewalllogs&amp;filename=$file&amp;limit=10&amp;state=any&amp;newcfg=$cfgname\">history</a> ";
		}
		print "/ <a href=\"$script?cmd=getonlineversion&amp;filename=$file&amp;directcall=$directcall\">get&nbsp;online&nbsp;version</a>\n";
		print " / <a href=\"$script?cmd=start\">httpmanage</a>\n";
		print "\n<br></small><br><!-- end_of_httpmanage -->";
		
		if( $directcall ) {
			# no, we're not included in some html/php file
			global $suffix_picture;
			$filestat = stat("$docroot$file");
			$filedate = $filestat[9];			# date of last modification
			$filedate = date("Y-m-d H:i:s", $filedate);
			$filesize = $filestat[7];			# size in bytes
			print "<br><strong>Size:</strong> $filesize bytes<br>\n";
			print "<strong>Last modified:</strong> $filedate<br>\n";
			if( ereg($suffix_picture, $file) ) {
				print "<strong>Image:</strong><br>\n<img src=\"$file\">";
			}
			footer();
		}

	} else if( $cmd == "start" ) {
		# the main screen (not included in any file, but accessed via the "httpmanage" link)
		headline("httpmanage");
		print '<p>httpmanage is a PHP3 tool that provides several convenience functions to manage two versions
			of WWW files. When configured right, links will show up in your html/php files that offer
			you these functions. Additionally there are:</p>';
		print '<ul><li>View changelogs (requires MySQL or PostgreSQL):<ul>';
		reset($config);
		while( list($pattern, $cfg) = each($config) ) {
			list($cfgfile, $tmpcfgname) = $cfg;
			print '<li><a href="admin.php3?cmd=viewalllogs&amp;limit=10&amp;distinct=on&amp;state=any&amp;newcfg='.$tmpcfgname.'">project \''.$tmpcfgname.'\'</a>';
		}
		print '</ul>';
		print '	<li>Go to a certain file (this can be any type of file in your webroot, give path
				relative to webroot, with a leading slash):
				<form action="admin.php3" method="get">
					Filename: <input type="text" name="filename" size="50">
					<input type="submit" value="Go">
				</form></ul>';
		print '<hr><i>httpmanage 0.4.3, <a href="http://mini.gt.owl.de/~dnaber/httpmanage/">http://mini.gt.owl.de/~dnaber/httpmanage/</a></i>';
		footer();
	
	# here are the document-specific actions:
	} else if( $cmd == "previewdev" ) {
		# don't show httpmanage links, since we want a real preview, i.e. do nothing
		print "";
		
	} else if( $cmd == "changestate" ) {
		# form to change state and add logfile entry
		global $statenow, $directcall;
		headline("change state");
		print "<form action=\"$script\" method=\"post\">";
		print '<input type="hidden" name="cmd" value="dochangestate">';
		print "<input type=\"hidden\" name=\"filename\" value=\"$file\">";
		print "<input type=\"hidden\" name=\"directcall\" value=\"$directcall\">";
		print 'State:<br>';
		print '<select name="newstate">';
		# get the "names" of the states (e.g. "in development" etc):
		$db->query("SELECT state.id, state.description FROM state");
		while( $db->next_record() ) {
			if( $statenow == $db->f("id") ) {
				$check = " selected";
			} else {
				$check = "";
			}
			print "<option value=\"" .$db->f("id"). "\"$check>" .$db->f("description"). "\n";
		}
		print '</select><br>';
		print 'User:<br><input type=text" name="user"><br>';
		print 'Comment about this change:<br><textarea name="comment" cols="70" rows="5"></textarea>';
		print '<br><input type="submit" value="change state"></form>';
		print "</html>";
		footer();

	} else if( $cmd == "dochangestate" ) {
		# actual action to change state and add logfile entry
		global $newstate, $comment, $user, $directcall;
		# get a verbose description of the change that we can put in the log:
		$query = "	SELECT state.description, state.id
				FROM state
				WHERE state.id = $newstate";
		$db->query($query);
		$db->next_record();
		$newstate_verbose = $db->f("description");
		$fileid = getfileid($file);
		$statenow = changefilestate($fileid, $newstate);
		$now = date("Y-m-d H:i:s");
		$statechange = "";
		if( $newstate != $statenow ) {
			$statechange = "(changed state to $newstate_verbose)\n";
		}
		$query = "	INSERT INTO log (fileid, date, entry, username)
				VALUES ($fileid, '$now', '$statechange$comment', '$user')";
		$db->query($query);
		header("Location: ".getbacklink($file, $directcall));
		
	} else if( $cmd == "putonline" ) {
		# confirmation if to put the file online
		global $uploadpassword, $directcall;
		headline("upload?");
		print "Upload ...$file to ...".filenamefilter($file)." on the server?";
		print "<form action=\"$script\" method=\"post\">\n";
		print '<input type="hidden" name="cmd" value="doputonline">'."\n";
		print '<input type="hidden" name="filename" value="'.$file.'">'."\n";
		print "<input type=\"hidden\" name=\"directcall\" value=\"$directcall\">";
		if( $uploadpassword ) {
			print 'Upload password: <input type="password" name="password"><br>';		
		}
		if( $usedb ) {
			print 'User: <input type=text" name="user"><br>';
			print 'Comment about this upload:<br><textarea name="comment" cols="70" rows="5"></textarea>';
		}
		print '<br><strong><input type="submit" name="submit" value="Upload"></strong> ';
		print '<input type="submit" name="submit" value="Cancel">';
		print "</form>\n";
		footer();

	} else if( $cmd == "doputonline" ) {
		# actually upload the file or return if the user chose Cancel
		global $submit, $password, $uploadpassword, $comment, $user, $directcall;
		if( $submit == "Cancel" ) {
			header("Location: $server$file");
			return;
		}
		headline("uploading");
		if( ! isset($password) ) {	# just avoid a warning
			$password = "";
		}
		$ret = upload($file, $password, $comment, $user);
		if( $ret == 1 ) {
			print "<a href=\"$onlineurl\">check uploaded file (maybe you need to reload that file)</a><br>";
			print '<a href="'.getbacklink($file, $directcall).'">back</a>';
		} else if( $ret == 2 )  {
			print "<br><strong>Error executing upload command, see above.</strong>";
		} else if( $ret == 3 )  {
			print "Error: password invalid - access denied.";
		}		
		footer();
			
	} else if( $cmd == "getonlineversion" ) {
		# confirmation to download the file from the online server
		global $downloadcmd, $downloaddir, $directcall;
		headline("get online version");
		$downloadcmd = ereg_replace("FROMFILE", filenamefilter($file), $downloadcmd);
		$downloadcmd = ereg_replace("DOWNLOADDIR", $downloaddir, $downloadcmd);
		$tmpfilename = ereg_replace("/", "_", $file).uniqid("");
		$downloadcmd = ereg_replace("TOFILE", $tmpfilename, $downloadcmd);
		$downloadcmd = EscapeShellCmd($downloadcmd)." 2>&1";
		print "<pre>";
		system($downloadcmd, $ret);
		print "</pre>";
		if( $ret == 0 ) {
			print "<br><a href=\"$script?cmd=dogetonlineversion&amp;tmpfilename=$tmpfilename&amp;directcall=$directcall&amp;filename=$file\">download now</a> -- ";
			print "<a href=\"$script?cmd=dontgetonlineversion&amp;filename=$file&amp;tmpfilename=$tmpfilename&amp;directcall=$directcall\">cancel/back</a>";
		} else {
			print "<br><strong>Error executing '$downloadcmd', see above (return value $ret).</strong>";
		}
		footer();

	} else if( $cmd == "dontgetonlineversion" ) {
		# user pressed cancel instead of download
		global $downloaddir, $tmpfilename, $directcall;
		if( file_exists("$downloaddir/$tmpfilename") ) {
			unlink("$downloaddir/$tmpfilename");
		}
		header("Location: ".getbacklink($file, $directcall));
	
	} else if( $cmd == "dogetonlineversion" ) {
		# actually download the file (not via http but, e.g., scp. this way SSI tags etc. are preserved)
		global $downloadcmd, $downloaddir, $tmpfilename;
		$onlinefile = getfile("$downloaddir/$tmpfilename");
		if( ! $onlinefile ) {
			headline("get online version - error");
			print "Got file from remote server but couldn't load it from '$downloaddir/$tmpfilename'?!";
			footer();
			return;
		}
		header("Content-Type: none/save_to_disk; name=$file");		# fixme: it's ugly, but we want the user to download the file...
		header("Content-Disposition: attachment; filename=$file");	# suggest the original filename
		print $onlinefile;
		unlink("$downloaddir/$tmpfilename");
		
	} else if( $cmd == "diff" ) {
		# show diff between development version and online version (both fetched via http)
		global $diffcmd, $difftype, $dos_unix_ignore, $scripttag, $diff_old_start, $diff_old_end, $diff_new_start, $diff_new_end, $downloaddir;
		headline("diff dev/online via http");
		print "<strong>Comparing with $diffcmd</strong>";
		$link = "$server$script?cmd=diff&amp;filename=$file&amp;difftype=$difftype";
		# get online file via http:
		$onlinefile = getfile($onlineurl);
		if( ! $onlinefile ) {
			print "<br>Error: Couldn't get '$onlineserver$file' via http. Maybe the file isn't online yet?";
			footer();
			return;
		}
		$uniqid = uniqid("");
		if( ! is_binary($onlineurl) ) {		# avoid trashing the file
			# ignore the difference caused by ourselfes:
			$onlinefile = ereg_replace($scripttag, '', $onlinefile);
		}
		# get local file via http:
		$devfile = getfile("$server$file");
		if( ! $devfile ) {
			print "Couldn't get '$server$file' via http";
			footer();
			return;
		}
		if( ! is_binary($file) ) {		# avoid trashing the file
			# ignore the difference caused by ourselfes:
			$devfile = ereg_replace("<!-- begin_of_httpmanage -->.*<!-- end_of_httpmanage -->", '', $devfile);
			# differences between unix and dos may be ignored: just replace "\r" by "":
			if( $dos_unix_ignore ) {
				$onlinefile = ereg_replace("\r", "", $onlinefile);
				$devfile = ereg_replace("\r", "", $devfile);
				print ", ignoring differences in DOS/Unix line endings. <a href=\"$link&amp;dos_unix_ignore=0\">Don't ignore</a>";
			} else {
				print ", differences in DOS/Unix line endings may make the whole file different. ";
				print "<a href=\"$link&amp;dos_unix_ignore=1\">Ignore these differences</a>";
			}
		}
		# save both files so diff can work on them:
		if( ! putfile("$downloaddir/httpmanage_onlinefile$uniqid", $onlinefile) ) {
			print "Couldn't write '$downloaddir/httpmanage_onlinefile$uniqid'";
			return;
		}
		if( ! putfile("$downloaddir/httpmanage_localfile$uniqid", $devfile) ) {
			print "Couldn't write '$downloaddir/httpmanage_localfile$uniqid'";
		}
		print "<br>${diff_old_start}- online version$diff_old_end (<a href=\"$onlineurl\">$onlineurl</a>): ".strlen($onlinefile)." bytes<br>";
		# todo: show last modified date for online version, too
		print "${diff_new_start}+ dev version$diff_new_end (<a href=\"$server$file\">$server$file</a>): ".strlen($devfile)." bytes";
		$filedate = stat("$docroot$file");
		$filedate = date("Y-m-d H:i:s", $filedate[9]);
		print ", changed $filedate<br><br>";
		$cmd = EscapeShellCmd("$diffcmd $downloaddir/httpmanage_onlinefile$uniqid $downloaddir/httpmanage_localfile$uniqid");
		exec($cmd, $arr, $ret);
		if( $ret == 0 ) {
			print "<p><strong>Files are identical.</strong></p>";
		} else if( $ret == 1 ) {
			if( $difftype == "wheter" ) {
				print "<p><strong>Files differ (no output of differences since file's suffix suggests it's binary).</strong></p>";
			} else {
				print "<p><strong>Differences between files:</strong></p>";
			}
		} else if( $ret >= 2 ) {
 			print "<p>Error executing '$cmd' (return value $ret)</p>";
      		}
 		if( $difftype != "wheter" ) {
 			# show the differences, not only wheter the files differ:
	 		print "<pre>";
 			for( $ct = 0; $ct < count($arr); $ct++ ) {
 				# escape '<' and '>' since we print html:
 				$arr[$ct] = ereg_replace('<', "&lt;", $arr[$ct]);
				$arr[$ct] = ereg_replace('>', "&gt;", $arr[$ct]);
				# set colors for changed lines:
				if( $ct >= 2 && ereg("^-", $arr[$ct]) ) {
					$start = $diff_old_start;
					$end = $diff_old_end;
				} elseif( $ct >= 2 && ereg('^\+', $arr[$ct]) ) {
					$start = $diff_new_start;
					$end = $diff_new_end;
				} else {
					$start = '';
					$end = '';
				}
				print $start.$arr[$ct].$end."\n";
			}
			print "</pre>";
		}
		unlink("$downloaddir/httpmanage_onlinefile$uniqid");
		unlink("$downloaddir/httpmanage_localfile$uniqid");
		footer();

	} else if( $cmd == "checksyntax" ) {
		# use some external program (e.g. weblint) to check html syntax
		global $checkcmd, $docroot;
		headline("check syntax");
		$cmd = EscapeShellCmd("$checkcmd $docroot$file")." 2>&1";
		exec($cmd, $arr, $ret);
		print "<pre>";
		for( $ct = 0; $ct < count($arr); $ct++ ) {
			# escape '<' and '>' because of html:
			$arr[$ct] = ereg_replace('<', "&lt;", $arr[$ct]);
			$arr[$ct] = ereg_replace('>', "&gt;", $arr[$ct]);
			print $arr[$ct]."\n";
		}				
		print "</pre>";
		if( $ret > 0 ) {
			print "Couldn't execute '$cmd', see above (return value $ret).";
		} else if( $ct == 0 ) {
			print "Syntax okay.";
		}
		footer();
	
	# these are the actions that aren't document specific:
	} else if( $cmd == "viewalllogs" ) {
		# show latest logfile entries and a form to filter them
		global $limit, $username, $state, $distinct, $newcfg;
		if( ! $limit ) {
			$limit = 10;
		}
		$query = "SELECT 	files.id AS fid, files.filename,
					log.date, log.entry, log.username,
					state.id, state.description, overview.state
					FROM files, state, overview, log
					WHERE	files.id = overview.fileid AND
						files.id = log.fileid AND
						overview.state = state.id ";
		if( $state != 'any' ) {
			$query .=		"AND overview.state = $state ";
		}
		if( isset($username) && $username ) {
			$query .=		"AND log.username = '$username' ";
		}
		if( isset($file) && $file && $file != $script ) {
			$query .=		"AND files.filename = '$file'";
		} else {
			$file = "";
		}
		$query .= "	ORDER BY log.date DESC";
		if( ! isset($distinct) ) {	# with distinct, limit is done manually, see below
			$query .= " LIMIT $limit";
		}
		headline("view $limit latest logfile entries - project '$cfgname'");
		print '<form action="'.$script.'" method="post">';
		print '<input type="hidden" name="cmd" value="putonlineall">';
		$ct = 0;
		$shown = array();
		$db->query($query);
		while( $db->next_record() ) {
			# todo: the distinct stuff should be possible in SQL, shouldn't it?
			if( ! my_in_array($db->f("fid"), $shown) ) {
				if( isset($distinct) ) {
					if( $ct >= $limit ) {
						break;
					}
					$shown = my_array_push($db->f("fid"), $shown);
				}
				$ct++;
				print "<strong>$ct. <input type=\"checkbox\" name=\"" .$db->f("fid"). "\">";
				print " <a href=\"$server" .$db->f("filename"). "\">" .$db->f("filename"). "</a></strong> ";
				print $db->f("date"). ", " .$db->f("username"). ", currently " .$db->f("description"). "<br>\n";
				print $db->f("entry");
				print "<hr>\n";
			}
		}
		if( $ct == 0 ) {
			print "<p>(no matches)</p>";
		} else if( $ct == $limit ) {
			print "(number of listed matches is limited)<hr>";
		}
		print '<table width="100%"><tr><td align="right"><input type="submit" value="Upload all selected files"></td></tr></table></form>';
		print "<form action=\"$script\" method=\"get\">";
		print '<strong>filter log:</strong><br>';
		print '<input type="hidden" name="cmd" value="viewalllogs">';
		print '<input type="hidden" name="filename" value="' .$file. '">';
		print '<input type="hidden" name="newcfg" value="' .$newcfg. '">';
		$file = isset($file) ? $file : "";			# avoid warning
		print 'Filename = <input type="text" name="filename" value="'.$file.'" size="30"> ';
		$username = isset($username) ? $username : "";		# avoid warning
		print 'AND user = <input type="text" name="username" value="'.$username.'" size="8"> ';
		print 'AND current state = <select name="state">';
		print '<option value="any">(any)';
		$db->query("SELECT state.id, state.description FROM state");
		while( $db->next_record() ) {
			if( $state == $db->f("id") ) {
				$sel = " selected";
			} else {
				$sel = "";
			}
			print "<option value=\"" .$db->f("id") ."\"$sel>" .$db->f("description"). "\n";
		}
		print '</select><br>';
		print 'max. matches: <input type="text" name="limit" value="'.$limit.'"size="3">';
		print ' <input type="checkbox" name="distinct"';
		if( isset($distinct) ) {
			print " checked";
		}
		print "> Distinct (don't list a file more than once)<br>";
		print '<input type="submit" value="View filtered logfile">';
		print '</form>';
		footer();

	} else if( $cmd == "putonlineall" ) {
		# confirmation of uploading all selected files
		global $uploadpassword;
		headline("upload selected files?");
		print "<form action=\"$script\" method=\"post\">\n";
		print '<input type="hidden" name="cmd" value="doputonlineall">'."\n";
		$ct = 0;
		while (list($id, $value) = each($GLOBALS["HTTP_POST_VARS"])) {
			if( ereg("^[0-9]+$", $id) && $value == 'on' ) {
				$query = "	SELECT files.filename, files.id
						FROM files
						WHERE files.id = $id";
				$db->query($query);
				$db->next_record();
				echo $db->f("filename") . "<br>\n";
				print '<input type="hidden" name="'.$id.'" value="on">';
				$ct++;
			}
		}
		if( $ct == 0 ) {
			print "Error: no files selected - nothing to upload\n";
			footer();
			return;
		}
		print "<br>";
		if( $uploadpassword ) {
			print 'Upload password: <input type="password" name="password"><br>';		
		}
		if( $usedb ) {
			print 'User: <input type=text" name="user"><br>';
			print 'Comment about this upload (will be added to every uploaded file):<br><textarea name="comment" cols="70" rows="5"></textarea>';
		}
		print '<br><strong><input type="submit" name="submit" value="Upload"></strong> ';
		print '<input type="submit" name="submit" value="Cancel">';
		print "</form>\n";
		footer();
		
	} else if( $cmd == "doputonlineall" ) {
		# upload all selected files at once
		global $password, $comment, $user;
		headline("uploading selected files");
		$ct = 0;
		while (list($var, $value) = each($GLOBALS["HTTP_POST_VARS"])) {
			if( ereg("^[0-9]+$", $var) ) {
				$query = "	SELECT files.filename, files.id
						FROM files
						WHERE files.id = $var";
				$db->query($query);
				$db->next_record();
				print "<br>Uploading <a href=\"" .$db->f("filename"). "\">" .$db->f("filename"). "</a> ";
				print " to <a href=\"" .$onlineserver.$db->f("filename"). "\">" .$onlineserver.$db->f("filename"). "</a><br>\n";
				flush();
				$ret = upload($db->f("filename"), $password, $comment, $user);
				if( $ret == 1 ) {
					$ct++;
				} else if( $ret == 2 )  {
					print "<br><strong>Error executing the upload command, see above (won't try to upload the rest of the files)</strong>";
					break;
				} else if( $ret == 3 )  {
					print "<br><strong>Error: password invalid - access denied (won't try to upload any other files)</strong>";
					break;
				}		
			}
		}
		print "<br><strong>Uploaded $ct files.</strong>";
		footer();
	
	} else {
		print "Error: command '$cmd' not implemented.";
	}
}

# --- helper functions -----------------------------------------

# this will be in php4 as in_array():
function my_in_array($val, $arr) {
	for( $ct = 0; $ct < count($arr); $ct++ ) {
		if( $val == $arr[$ct] ) {
			return 1;
		}
	}
	return 0;
	
}

# this will be in php4 as array_push():
function my_array_push($val, $arr) {
	$size = count($arr);
	$arr[$size] = $val;
	return $arr;
}

# create an url that points back to the file we're working on:
function getbacklink($file, $directcall) {
	global $script, $server;
	# $server is added because this can be used in a 'Location:' redirect:
	if( $directcall ) {
		return "$server$script?filename=$file";
	} else {
		return "$server$file";	
	}
}

# we just look at the filename to see if we might be handling a binary file:
function is_binary($filename) {
	global $suffix_html;
	if( ereg($suffix_html, $filename) ) {
		return 0;
	} else {
		return 1;
	}
}

# get fileid if file exists in table 'files'. otherwise, add file:
function getfileid($file) {
	global $db;
	$query = "	SELECT files.id, files.filename
			FROM files
			WHERE files.filename = '$file'";
	$db->query($query);
	$db->next_record();
	if( ! $db->f("id") ) {	# file not yet in table files
		$query = "INSERT INTO files (filename) VALUES ('$file')";
		$db->query($query);
		$query = "	SELECT files.id, files.filename
				FROM files
				WHERE files.filename = '$file'";
		$db->query($query);
		$db->next_record();
		if( ! $db->f("id") ) {		# file still not in table?!?!
			print "Error: Couldn't add '$file' to table 'files' using query: $query";
			return;
		}
	}
	return $db->f("id");
}

# update/insert the state of a file:
function changefilestate($fileid, $newstate) {
	global $db;
	$query = "SELECT overview.state, overview.fileid FROM overview WHERE overview.fileid = $fileid";
	$db->query($query);
	$ct = 0;
	while( $db->next_record() ) {
		$oldstate = $db->f("state");
		$ct++;
	}
	if( $ct == 0 ) {	# file is not yet in table overview
		$query = "	INSERT INTO overview (fileid, state)
				VALUES ($fileid, $newstate)";
				$statenow = -1;
	} else {
		$query = "	UPDATE overview
				SET state = $newstate
				WHERE fileid = $fileid";
	}
	$db->query($query);
	return $oldstate;
}

# upload a file to another server, typically using scp. returns:
# 1 for success, 2 for error, 3 for access denied (wrong httpmanage password)
function upload($file, $password, $comment, $user) {
	global $db, $uploadpassword, $uploadcmd, $usedb;
       	if( $uploadpassword && ($uploadpassword != crypt($password, substr($uploadpassword, 0, 2))) ) {
       		return 3;
       	}
       	# todo: get rid of '//' (use '/' instead)
       	$tempuploadcmd = ereg_replace("FROMFILE", $file, $uploadcmd);
       	$tempuploadcmd = ereg_replace("TOFILE", filenamefilter($file), $tempuploadcmd);
       	$tempuploadcmd = EscapeShellCmd($tempuploadcmd)." 2>&1";
       	print "<pre>";
		system($tempuploadcmd, $ret);
       	print "</pre>\n";
       	if( $ret == 0 ) {
       		if( $usedb ) {
        		global $docroot;
        		$fileid = getfileid($file);
        		changefilestate($fileid, 0);
        		$filedate = stat("$docroot$file");
        		$filedate = date("Y-m-d H:i:s", $filedate[9]);	# date of last modification
        		$query = "	UPDATE files
        				SET date = '$filedate'
        				WHERE id = $fileid";
        		$db->query($query);
        		$now = date("Y-m-d H:i:s");
        		$query = "	INSERT INTO log (fileid, date, username, entry)
        				VALUES ($fileid, '$now', '$user', '(file upload) $comment')";
        		$db->query($query);
        	}
       		return 1;
       	} else {
       		return 2;
       	}
}

# translate the filename to fit the other server:
function filenamefilter($filename) {
	global $filename_replace, $filename_replace_with;
	if( ! $filename_replace ) {
		return $filename;
	}
	$filename = ereg_replace($filename_replace, $filename_replace_with, $filename);
	return $filename;
}

# translate the url to fit the other server:
function urlfilter($url) {
	global $url_replace, $url_replace_with;
	if( ! $url_replace ) {
		return $url;
	}
	$url = ereg_replace($url_replace, $url_replace_with, $url);
	return $url;
}

# print out html header
function headline($title) {
	global $file, $cfgname, $script;
	print '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">';
	print "\n<html>\n<head>\n";
	print "<title>$title: $file in project '$cfgname'</title>\n</head>\n<body>\n";
	print "<h1>$title</h1>\n";
}

# print out html footer:
function footer() {
	print "\n</body>\n</html>\n";
}

# fetch file contents or return nothing in case of error
function getfile($filename) {
	$fp = fopen($filename, "r");
	if( ! $fp ) {
		fclose($fp);
		return;
	}
	$content = "";
	while( $str = fread($fp, 1000) ) {
		$content .= $str;
	}
	fclose($fp);
	return $content;	
}	

# save file; return 0 in case of error, 1 in case of success
function putfile($filename, $content) {
	$fp = fopen($filename, "w");
	if( ! $fp ) {
		//print "Couldn't open '$filename' for writing";
		return 0;
	}
	if( ! fwrite($fp, $content) ) {
		//print "Couldn't write to '$filename'";
		return 0;
	}
	fclose($fp);
	return 1;	
}	

# check (using regex) if REMOTE_ADDR is listed in $ips
function checkaccess($ips) {
	if( ! $GLOBALS["REMOTE_ADDR"] ) {
		print "<br>[httpmanage: Couldn't find your IP address in REMOTE_ADDR -- access denied]";
		return false;
	}
	$access = 0;
	reset($ips);
	$ct = 0;
	while( $ip = $ips[$ct] ) {
		if( ereg("^$ip", $GLOBALS["REMOTE_ADDR"]) ) {
			$access = 1;
			break;
		}
		$ct++;
	}
	if( $access == 1 ) {
		return true;
	} else {
		return false;
	}
}

# --- main program ------------------------------------------------------------

# find out which file we are expected to work on (should work no matter if this script is
# included via PHP's include() or via SSI:
if( isset($filename) ) {
	$file = $filename;
} else {
	$file = $GLOBALS["DOCUMENT_URI"];
	if( ! $file ) {
		$file = $GLOBALS["SCRIPT_NAME"];
	}
}

# lookup the external configuration file for the file:
$foundcfg = 0;
reset($config);
while( list($pattern, $cfg) = each($config) ) {
	list($cfgfile, $cfgname) = $cfg;
	if( isset($newcfg) ) {
		if( $cfgname == $newcfg ) {
			$foundcfg = 1;
			break;
		}
     	} else {
		if( ereg($pattern, $file) ) {
      			$foundcfg = 1;
      			break;
      		}
      	}
}
if( ! $foundcfg ) {
	print "httpmanage error: Couldn't choose external configuration - no pattern match in \$config ";
	print "(or maybe \$ssi_include isn't set correctly).";
	exit;
} else {
	if( ! file_exists($cfgfile) ) {
		print "httpmanage error: config file '$cfgfile' not found!";
		exit;
	}
	include($cfgfile);
}

# we can't check earlier because the configuration saying which IPs are allowed
# to access has to be loaded first:
if( checkaccess($ips) ) {
	set_time_limit($timelimit);
	$db = new DB_httpmanage;
	action($file);
}
# else: do nothing, people/IPs not allowed to use this script won't even notice it

?>
